library Super_wdx;

uses
  SysUtils, ContPlug, inifiles, windows, Classes, Messages,
  StrUtils,
  SyncObjs,
  WdxFieldsProc in 'WdxFieldsProc.pas';

{$R *.res}
{$E wdx}
{$IFDEF WIN64}
  {$E w64}
  {$IFNDEF VER230}
    {$E wdx64}
  {$ENDIF}
{$ENDIF}
{$WEAKLINKRTTI ON}

var
  IniPath: string;
  ini: TIniFile;
  tcdir: string;
  basepath: string;
  RTL: TCriticalSection;

function GetPluginPath(PluginName: string): string;
begin
  Result := AnsiReplaceStr(ini.ReadString('Plugins', PluginName, ''),
    '%commander_path%', tcdir);
end;

// ------------------------------------------------------------------------------
procedure ReadLoadingConfiguration(const ini: TIniFile); inline;
begin
  if not Assigned(ini) then
    Exit;
  ShowError := ini.ReadBool('Super', 'ShowError', false);
  DateTimeISO := ini.ReadBool('Super', 'DateTimeISO', DateTimeISO);
  Debug := ini.ReadBool('Super', 'Debug', false);
  EmptyReplace := ini.ReadBool('Super', 'EmptyReplace', false);
  CustomFoldersView := ini.ReadInteger('Super', 'CustomFoldersView', 0);
  CustomFolderFileName := ini.ReadString('Super', 'CustomFolderFileName',
    'folder.ini');
  TextInFloatFields := ini.ReadInteger('Super', 'TextInFloatFields', TextInFloatFields);
end;

// ------------------------------------------------------------------------------
function GetFormattedString(ConfigString: string; filename: string): string;
var
  res: string;
  i: integer;
  FieldNum, UnitNum { ,UnitType } : byte;
  bufmode, fieldmode, dotmode, PlugMode: boolean; // close/open
  plugin, field, dot: string;
  unformatted_symbols: integer; // counter of symbols not in %bla@bla.wdx%
  ConString: PChar;
begin
  try
    unformatted_symbols := 0;
    bufmode := false;
    fieldmode := false;
    plugmode := false;
    dotmode := false;
    plugin := '';
    field := '';
    ConString := Pointer(ConfigString);
    for i := 0 to length(ConString) - 1 do
    begin
      if ConString[i] = '%' then
      begin
        bufmode := not bufmode;
        if bufmode and not fieldmode then
          fieldmode := True;
      end;
      if bufmode and fieldmode and not PlugMode and (ConString[i] = '.') then
        dotmode := True;
      if bufmode and fieldmode and (ConString[i] = '@') then
      begin
        fieldmode := false;
        dotmode := false;
        PlugMode := True;
      end;
      { ----------------------------------------------------------------------------- }
      if fieldmode and not dotmode then
        field := field + ConString[i];
      if fieldmode and dotmode then
        dot := dot + ConString[i];
      if PlugMode then
        plugin := plugin + ConString[i];
      if (not bufmode) and (ConString[i] <> '%') then
      begin
        res := res + ConString[i];
        inc(unformatted_symbols);
      end;
      { ----------------------------------------------------------------------------- }
      if (not bufmode) and (ConString[i] = '%') then
      begin
        bufmode := false; // ALL MODES MUST BE FALSE
        PlugMode := false;
        delete(plugin, 1, 1);
        delete(plugin, length(plugin), 1);
        delete(field, 1, 1);
        if length(dot) > 1 then
          delete(dot, 1, 1);
        if {$IFDEF WIN32} (lowercase(plugin) = 'super_wdx.wdx') {$ENDIF}
          {$IFDEF WIN64}(lowercase(plugin) = 'super_wdx.wdx64') {$ENDIF} or
          (lowercase(field) = 'super_column') then
        begin
          DebugLog('Hmmmm... Don`t try use super_wdx in super_wdx, it cause to errors.'
            + #10 + #13 +
            '  super_wdx   ,  .');
          res := ''
        end
        else
        begin
          plugin := GetPluginPath(plugin);
          {$IFDEF WIN64}
          if AnsiSameText(ExtractFileExt(plugin), '.wdx') then
            plugin := plugin + '64';
          {$ENDIF}
          FieldNum := GetWdxFieldNum(plugin, field, dot, UnitNum);
          res := res + GetWdxField(plugin, filename, FieldNum, UnitNum);
          plugin := '';
          field := '';
          dot := '';
        end;
      end;
    end;
    if (length(res) <= unformatted_symbols) and EmptyReplace then
    begin
      Result := ExtractFileName(filename);
    end
    else
    begin
      Result := res;
    end;
  except
    on e: exception do
    begin
      DebugLog('Some malfunction in procedure GetFormattedString.' + #10 + #13 +
        '  GetFormattedString.' + #10 + #13 + 'Variable values:' + #10
        + #13 + 'ConString: ' + ConfigString + #10 + #13 + 'res: ' + res + #10 +
        #13 + 'plugin: ' + plugin + #10 + #13 + 'field: ' + field);
      Result := 'ERROR!';
    end;
  end;
end;

// ------------------------------------------------------------------------------
function GetColumnValue(filename: string; ColumnNum, Flags: byte): string;
var
  ContentString: string;
  fileext: string;
  CustomIni: tinifile;
  ColumnDelay: byte;
  IsDir: Boolean;
begin
  IsDir := DirectoryExists(filename);
  if IsDir then
    fileext := '.<DIR>'
  else
    fileext := ExtractFileExt(filename);
  if fileext = '' then
  begin
    if IsDir then
    begin
      fileext := '.<DIR>'
    end
    else
    begin
      Result := '';
      exit;
    end;
  end;
  delete(fileext, 1, 1);
  ini := tinifile.Create(IniPath);
  try
    ReadLoadingConfiguration(ini);
    ColumnDelay := ini.ReadInteger('Column #' + IntToStr(ColumnNum),
      'DelayMode', 0);

    case CustomFoldersView of
      1:
        begin
          if Fileexists(IncludeTrailingPathDelimiter(ExtractFilePath(filename)) +
            CustomFolderFileName) then
          begin
            CustomIni := TIniFile.Create
              (IncludeTrailingPathDelimiter(ExtractFilePath(filename)) +
              CustomFolderFileName);
            try
              ColumnDelay := CustomIni.ReadInteger('Column #' + IntToStr(ColumnNum),
                'DelayMode', 0);
            finally
              CustomIni.Free;
            end;
          end
          else
            ColumnDelay := ini.ReadInteger('Column #' + IntToStr(ColumnNum),
              'DelayMode', 0);
        end;
      2:
        begin
          // NO MORE SUPPORTED
        end;
      3:
        begin
          // NO MORE SUPPORTED
        end;
    end;

    if (Flags = CONTENT_DELAYIFSLOW) and (ColumnDelay <> 0) then
    begin
      case ColumnDelay of
        1:
          begin
            Result := 'ft_delayed';
            exit;
          end;
        2:
          begin
            Result := 'ft_ondemand';
            exit;
          end;
      end;
    end
    else

      case CustomFoldersView of
        1:
          begin
            if Fileexists(IncludeTrailingPathDelimiter(ExtractFilePath(filename)) +
              CustomFolderFileName) then
            begin
              CustomIni := tinifile.Create
                (IncludeTrailingPathDelimiter(ExtractFilePath(filename)) +
                CustomFolderFileName);
              try
                ContentString := CustomIni.ReadString
                  ('Column #' + IntToStr(ColumnNum), fileext, '');
                if (ContentString = '') and (ini.ReadBool('Super', 'Usedef', false))
                then
                begin
                  ContentString := CustomIni.ReadString
                    ('Column #' + IntToStr(ColumnNum), '<DEF>', '');
                end;
              finally
                CustomIni.Free;
              end;
            end
            else
              ContentString := ini.ReadString('Column #' + IntToStr(ColumnNum),
                fileext, '');
          end;
        2:
          begin
            // NO MORE SUPPORTED
          end;
        3:
          begin
            // NO MORE SUPPORTED
          end;
      else
        ContentString := ini.ReadString('Column #' + IntToStr(ColumnNum),
          fileext, '');
      end; // case

    if ContentString = '' then
      ContentString := ini.ReadString('Column #' + IntToStr(ColumnNum),
        fileext, '');
    if (ContentString = '') then
    begin
      if (ini.ReadBool('Super', 'Usedef', false)) then
      begin
        ContentString := ini.ReadString('Column #' + IntToStr(ColumnNum),
          '<DEF>', '');
        if ContentString = '' then
        begin
          Result := '';
          exit;
        end;
      end
      else
      begin
        Result := '';
        exit;
      end;
    end;
    Result := GetFormattedString(ContentString, filename);
  finally
    FreeAndNil(ini);
  end;
end;

// ------------------------------------------------------------------------------
function ContentGetSupportedField(FieldIndex: integer; FieldName: PAnsiChar;
  Units: PAnsiChar; maxlen: integer): integer; stdcall;
var
  ini: TIniFile;
  NOfCol: Integer;
begin
  ini := TIniFile.Create(IniPath);
  try
    NOfCol := ini.ReadInteger('SUPER', 'CustomColumns', 3);
    if NOfCol > 100 then NOfCol := 100;
    Units[0] := #0;
    if NOfCol > FieldIndex then
    begin
      StrLCopy(FieldName, PAnsiChar(AnsiString(
        'SUPER_COLUMN #' + IntToStr(FieldIndex + 1))), maxlen - 1);
      Result := ft_StringW;
    end
    else
      Result := 0;
  finally
    ini.Free;
  end;
end;

{ ------------------------------------------------------------------------------ }
//    1.5  Custom Columns View
function ContentGetValue(filename: pAnsichar; FieldIndex, UnitIndex: integer;
  FieldValue: pbyte; maxlen, Flags: integer): integer; stdcall;
begin
  Result := 0;
end;

function ContentGetValueW(filename: PChar; FieldIndex, UnitIndex: integer;
  FieldValue: pbyte; maxlen, Flags: integer): integer; stdcall;
var
  z: string;
begin
  try
    RTL.Enter;
    try
      z := GetColumnValue(filename, FieldIndex + 1, Flags);
      StrLCopy(PWideChar(FieldValue), PWideChar(z), maxlen div 2 - 1);
      Result := ft_StringW;
      if z = 'ft_delayed' then
      begin
        //fillchar(FieldValue^, maxlen, 0);
        Result := ft_delayed;
        exit;
      end else if z = 'ft_ondemand' then
      begin
        //fillchar(FieldValue^, maxlen, 0);
        Result := ft_ondemand;
        exit;
      end;
    finally
      RTL.Leave;
    end;
  except
    Result := 0;
  end;
end;

//    2.1   GetFormattedValue
function GetFormattedValueW(filename, ContentString: PWideChar;
  FieldValue: PWideChar; maxlen: integer): integer; stdcall;
begin
  Result := 0;
  ini := tinifile.Create(IniPath); // need create global ini object
  try
    ReadLoadingConfiguration(ini);
    StrLCopy(FieldValue, PWideChar(GetFormattedString(ContentString, filename)),
      maxlen div 2 - 1);
    Result := 1;
  finally
    ini.Free;
  end;
end;

function GetFormattedValue(filename, ContentString: pAnsichar; FieldValue: pAnsichar;
  maxlen: integer): integer; stdcall;
begin
  ini := tinifile.Create(IniPath); // need create global ini object
  try
    ReadLoadingConfiguration(ini);
    StrLCopy(FieldValue, PAnsiChar(AnsiString(GetFormattedString(ContentString, filename))),
      maxlen - 1);
    Result := 1;
  finally
    ini.Free;
  end;
  {
  Result := GetFormattedValueW(PWideChar(WideString(AnsiString(filename))),
    PWideChar(WideString(AnsiString(ContentString))),
    PWideChar(WideString(AnsiString(FieldValue))), maxlen);
  }
end;

procedure ContentPluginUnloading; stdcall;
begin
  try
    SelfUnloading;
  finally
    FreeAndNil(RTL);
  end;
end;

procedure ContentSetDefaultParams(dps: pContentDefaultParamStruct); stdcall;
var
  ini: TIniFile;
begin
  TC_ContentIni := dps.DefaultIniName;
  if not Assigned(RTL) then
    RTL := TCriticalSection.Create;

  ini := TIniFile.Create(IniPath);
  try
    AllowUnloadPlugins := ini.ReadBool('Super', 'AllowUnloadPlugins', AllowUnloadPlugins);
  finally
    ini.Free;
  end;
end;

exports
  ContentGetSupportedField,
  ContentGetValue,
  ContentGetValueW,
  GetFormattedValue,
  GetFormattedValueW,
  ContentPluginUnloading,
  ContentSetDefaultParams;

procedure DLLEntryProc(EntryCode: integer);
begin
  case EntryCode of
    DLL_PROCESS_ATTACH:
    begin
      PluginPath := GetModuleName(HInstance);
      IniPath := (IncludeTrailingPathDelimiter(ExtractFilePath(PluginPath)) +
        'super_wdx.ini');
      tcdir := AnsiLowerCase(GetEnvironmentVariable('commander_path'));
      basepath := (IncludeTrailingPathDelimiter(ExtractFilePath(PluginPath)) +
        'dirdata.zip');
    end;
    DLL_PROCESS_DETACH:
    begin
    end;
  end;
end;

begin
  //ReportMemoryLeaksOnShutdown := True;
  DLLProc := @DLLEntryProc;
  DLLEntryProc(DLL_PROCESS_ATTACH);
end.
